clear, clc, close all

%% Section 2: Epipolar Geometry-Consistent ACs

syms x1 x2 y1 y2 a1 a2 a3 a4 t1 t2

% Affine correspondence
x = [x1; x2]; y = [y1; y2]; A = [a1 a3; a2 a4]; t = [t1;t2];

%Essential matrix
syms e1 e2 e3 e4 e5 e6 e7 e8 e9
E = [e1 e2 e3; e4 e5 e6; e7 e8 e9].';

ACconsistEhomog = [x1*y1, x1*y2, x1, x2*y1, x2*y2, x2, y1, y2, 1
                a3*x1, a4*x1, 0, y1+a3*x2, y2+a4*x2, 1, a3, a4, 0
                y1+a1*x1, y2+a2*x1, 1, a1*x2, a2*x2, 0, a1, a2, 0] * E(:);
            
eq2 = simplify(subs(ACconsistEhomog, y, A*x+t));

[Mtmp aux] = equationsToMatrix(eq2,[A(:);t].');

M = [Mtmp -aux]; M = simplify(M);
m = [A(:);t];

disp(['Matrix M only depends on E and x:'])
disp(M)

%Check Equation 3:
assert(all(simplify(M*[m;1] - eq2) == 0))

%Compute nullspace of M:
nullSpaceM = simplify(null(M));

disp(['Matrix N in Equation 4:'])
disp(nullSpaceM(1:6,:))


%%%%% Assuming point correspondence (x,y) is known: %%%%%
eq5_aux = eq2(2:end);

eq5_aux = simplify(subs(eq5_aux, t, y-A*x));
[Ptmp aux] = equationsToMatrix(eq5_aux,A(:).');
P = [Ptmp -aux];

disp(['Matrix P only depends on E and (x,y):'])
disp(P)

%Check Equation 5:
assert(all(simplify(P*[A(:);1] - eq5_aux) == 0))

%Compute nullspace of P:
nullSpaceP = simplify(null(P));

disp(['Matrix Q in Equation 7:'])
Q = nullSpaceP(1:4,:);
disp(Q)

%% Section 2.1 - Extension to the Multi-view Case

%%% 2-DoF Formulation %%%
%For the sake of simplicity, assume QA, QB and QC are known and can be
%written as
syms qA1 qA2 qA3 qB1 qB2 qB3 qC1 qC2 qC3
QA = [qA1 0 qA2; 1 0 0; 0 qA1, qA3; 0 1 0];
QB = subs(QA, [qA1 qA2 qA3], [qB1 qB2 qB3]);
QC = subs(QA, [qA1 qA2 qA3], [qC1 qC2 qC3]);

syms z1 z2 alpha1 alpha2 beta1 beta2 gamma1 gamma2
z = [z1; z2]; alpha = [alpha1; alpha2]; beta = [beta1; beta2]; gamma = [gamma1; gamma2];

Avec = QA * [alpha;1];
Bvec = QB * [beta;1];
Cvec = QC * [gamma;1];


eq8homog = reshape(Cvec, [2 2]) - reshape(Bvec, [2 2])*reshape(Avec, [2 2]);
[S s] = equationsToMatrix(eq8homog,[beta; gamma].');
S = simplify(S);

sol8 = S\s;

disp('Expressions for variables in Equation 8:')
disp(collect(sol8,[alpha1 alpha2]))


% Using relationship between alpha and gamma, rewrite C:
syms lambda8 lambda9 lambda10

Cvec_ = subs(Cvec,[gamma], [lambda8*alpha1+lambda9; lambda8*alpha2+lambda10]);

[CM Cv] = equationsToMatrix(Cvec_,alpha.');

%Check correctness of last 4 rows of matrix L:

assert(all(all(simplify([CM -Cv] - [lambda8*QC(:,1:2) QC*[lambda9 lambda10 1].'])==0)))

